/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SPEECHENGINE_H
#define SPEECHENGINE_H

#include <string>
#include <functional>
#include <vector>
#include "error.h"

namespace ai_engine {
namespace lm {
namespace speech {

enum class ResultType {
    ContinuousRecognitionResult,
    OnceRecognitionResult,
    ContinuousSynthesisResult,
    OnceSynthesisResult,
    KeywordSpotterResult
};

struct RecognitionResult {
    std::string text;
    int errorCode;
    ResultType type;
};

struct SynthesisResult {
    std::string audioFormat;
    int audioRate;
    int audioChannel;
    std::vector<char> audioData;
    int errorCode;
    ResultType type;
};

using RecognitionResultCallback = std::function<void(RecognitionResult result)>;
using SynthesisResultCallback = std::function<void(SynthesisResult result)>;
using FinishedCallback = std::function<void(ResultType type)>;

class AbstractSpeechEngine
{
public:
    virtual ~AbstractSpeechEngine() {};
    virtual std::string backendName() = 0;
    virtual bool isCloud() = 0;

    virtual void setRecognizingCallback(RecognitionResultCallback callback) = 0;
    virtual void setRecognizedCallback(FinishedCallback callback) = 0;

    virtual void setSynthesizingCallback(SynthesisResultCallback callback) = 0;
    virtual void setSynthesizedCallback(FinishedCallback callback) = 0;

    // 流式语音识别
    virtual ErrorCode initContinuousRecognitionModule() = 0;
    virtual ErrorCode startContinuousRecognition(const std::string &params) = 0;
    virtual ErrorCode writeContinuousRecognitionAudioData(const std::vector<char> &audioData) = 0;
    virtual ErrorCode stopContinuousRecognition() = 0;
    virtual ErrorCode destroyContinuousRecognitionModule() = 0;
    
    // 非流式语音识别
    virtual ErrorCode initRecognizeOnceModule() = 0;
    virtual ErrorCode recognizeOnce(const std::string &params, const std::vector<char> &audioData) = 0;
    virtual ErrorCode destroyRecognizeOnceModule() = 0;

    // 流式语音合成
    virtual ErrorCode initContinuousSynthesisModule() = 0;
    virtual ErrorCode startContinuousSynthesis(const std::string &params) = 0;
    virtual ErrorCode writeContinuousSynthesisText(const std::string &text) = 0;
    virtual ErrorCode stopContinuousSynthesis() = 0;
    virtual ErrorCode destroyContinuousSynthesisModule() = 0;

    // 非流式语音合成
    virtual ErrorCode initSynthesizeOnceModule() = 0;
    virtual ErrorCode synthesizeOnce(const std::string &params, const std::string &text) = 0;
    virtual ErrorCode destroySynthesizeOnceModule() = 0;
};

}
}
}


#endif // SPEECHENGINE_H
